"""Run a OpenFOAM command"""

import sys
import string
from os import path

from FoamThread import FoamThread

class BasicRunner:
    """Base class for the running of commands

    When the command is run the output is copied to a LogFile and
    (optionally) standard-out

    The argument list assumes for the first three elements the
    OpenFOAM-convention:

    <cmd> <dir> <case>

    The directory name for outputs is therefor created from <dir> and
    <case>
    
    Provides some handle-methods that are to be overloaded for
    additional functionality"""
    
    def __init__(self,argv=None,silent=False,logname="PyFoam"):
        """argv - list with the tokens that are the command line
        if not set the standard command line is used
        
        silent - if True no output is sent to stdout
        logname - name of the logfile"""
        
        if argv==None:
            self.argv=sys.argv[1:]
        else:
            self.argv=argv

        self.silent=silent
        self.cmd=string.join(self.argv," ")
        self.dir=path.join(self.argv[1],self.argv[2])
        self.logFile=path.join(self.dir,logname+".logfile")

        self.run=FoamThread(self.cmd)

    def start(self):
        """starts the command and stays with it till the end"""
        
        fh=open(self.logFile,"w")

        self.startHandle()
        
        self.run.start()

        while self.run.check():
            self.run.read()
            line=self.run.getLine()

            if not self.silent:
                print line

            self.lineHandle(line)
            
            fh.write(line+"\n")
            fh.flush()

        self.stopHandle()
        fh.close()

    def startHandle(self):
        """to be called before the program is started"""
        pass
    
    def stopHandle(self):
        """called after the program has stopped"""
        pass
    
    def lineHandle(self,line):
        """called every time a new line is read"""
        pass
    
