"""Thread wrappers for OpenFOAM"""
from threading import Thread,Lock
from popen2 import Popen4

from PyFoam.Basics.LineReader import LineReader

class FoamThread(Thread):
    """Thread running an OpenFOAM command

    The output of the command can be accessed in a thread-safe manner,
    line by line

    Designed to be used by the BasicRunner-class"""
    
    def __init__(self,cmdline):
        """cmdline - Command line of the OpenFOAM command"""
        Thread.__init__(self)
        self.cmdline=cmdline
        self.output=None
        self.reader=LineReader()

        self.stateLock=Lock()
        self.setState(False)

        self.status=None

        self.lineLock=Lock()
        self.line=""

        self.stateLock.acquire()
        
    def run(self):
        """start the command"""
        # print "Starting ",self.cmdline
        run=Popen4(self.cmdline)
        self.output=run.fromchild

        self.hasSomethingToSay=True
        self.stateLock.release()

        self.status=run.wait()
        # print "Returned",self.status

    def read(self):
        """read another line from the output"""
        self.setState(self.reader.read(self.output))
        self.lineLock.acquire()
        self.line=self.reader.line
        self.lineLock.release()

    def getLine(self):
        """gets the last line from the output"""
        self.lineLock.acquire()
        val=self.line
        self.lineLock.release()

        return val
    
    def setState(self,state):
        """sets the state of the thread (is there any more output)"""
        self.stateLock.acquire()
        self.hasSomethingToSay=state
        #        print "Set: ",state
        self.stateLock.release()
        
    def check(self):
        """returns False if there is no more output of the command"""
        self.stateLock.acquire()
        state=self.hasSomethingToSay
        #        print "Get: ",state
        self.stateLock.release()

        return state 
    
