"""Working with parameter-files"""

import re,os
from os import path
from tempfile import mkstemp

from PyFoam.Basics.LineReader import LineReader
from FileBasis import FileBasis

class ParameterFile(FileBasis):
    """Represents a OpenFOAM parameter file"""
    
    def __init__(self,name,backup=False):
        """name - The name of the parameter file
        backup - create a backup-copy of the file"""
        FileBasis.__init__(self,name)

        if backup:
            self.backupName=self.name+".backup"
            self.execute("cp "+self.name+" "+self.backupName)
        else:
            self.backupName=None

    def restore(self):
        """if a backup-copy was made the file is restored from this"""
        if self.backupName!=None:
            self.execute("cp "+self.backupName+" "+self.name)
            
    def parameterPattern(self,parameter):
        """creates a regular expression that looks for aparameter

        parameter - name of the parameter"""
        return re.compile("(.*) *"+parameter+" +(.*) *;(.*)")
        
    def readParameter(self,parameter):
        """reads the value of a parameter

        parameter - name of the parameter"""
        exp=self.parameterPattern(parameter)
        
        l=LineReader()
        self.openFile()

        erg=""
        
        while l.read(self.fh):
            m=exp.match(l.line)
            if m!=None:
                if m.group(1).find(self.removedString)<0:
                    erg=m.group(2)
                    break

        self.closeFile()
        return erg
    
    def replaceParameter(self,parameter,newval):
        """writes the value of a parameter

        parameter - name of the parameter
        newval - the new value"""
        exp=self.parameterPattern(parameter)

        l=LineReader()
        self.openFile()

        (fh,fn)=self.makeTemp()

        while l.read(self.fh):
            toPrint=l.line

            m=exp.match(l.line)
            if m!=None:
                if m.group(1).find(self.removedString)<0:
                    toPrint =self.removedString+l.line+"\n"
                    toPrint+=parameter+" "+newval+"; "+self.addedString
            fh.write(toPrint+"\n")

        self.closeFile()
        fh.close()
        os.rename(fn,self.name)
